#
# btrfssubvolume.py: Installation step to create a BTRFS subvolume of the root filesystem
# so that it can be recovered if later changes corrupt the volume
#
# Kevin Lyons <kevin.x.lyons@oracle.com>
#
# Copyright 2012 Oracle
#
# This software may be freely redistributed under the terms of the GNU
# library public license.
#
# You should have received a copy of the GNU Library Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#import string
#import isys
#import resource
#import posix
#import stat
#import errno
#import parted
#import sys
#import struct
#import string
#import partitioning
#import partedUtils
#import raid
#import lvm
#import types
#import sha
#import crypt
#import random
import re
import os
import iutil
import gettext
from constants import *
from flags import flags


import logging
log = logging.getLogger("anaconda")

def doBTRFSSubvolume(anaconda):
    if anaconda.dir == DISPATCH_BACK:
        rc = anaconda.intf.messageWindow(_("Warning"),
                _("The installation has already begun. You "
                  "cannot go back past this point.\n\nWould you like to "
                  "continue with the installation?"),
                type="custom", custom_icon=["error","error"],
                custom_buttons=[_("_Exit installer"), _("_Continue")])

        if rc == 0:
            sys.exit(0)

        return DISPATCH_FORWARD

    # Determine if root is btrfs
    out = iutil.execWithCapture("mount", [], stderr="/dev/tty5", root = anaconda.rootPath)
    rex = re.compile("/ type btrfs")
    if rex.search(out):
        #btrfs subvolume create /install
        iutil.execWithRedirect("/sbin/btrfs", ["subvolume", "snapshot", "/",
            "/install"], stdout="/dev/tty5", stderr="/dev/tty5", root = anaconda.rootPath)

        #have to list the subvolumes to get the ID for install
        out = iutil.execWithCapture("/sbin/btrfs", ["subvolume", "list", "/"],
            stderr="/dev/tty5", root = anaconda.rootPath)
        log.info("Created BTRFS subvolume for install")
        rex = re.compile('ID [0-9]+')
        subvolID = rex.findall(out)[0].replace("ID ", "")

        #do a setdefault of this one
        iutil.execWithRedirect("/sbin/btrfs", ["subvolume", "set-default", subvolID, "/"],
            stdout="/dev/tty5", stderr="/dev/tty5", root = anaconda.rootPath)

