from installclass import BaseInstallClass
from constants import *
from flags import flags
from product import *
import os
import iutil
import types
try:
    import wibbleinstnum
except ImportError:
    instnum = None
import gettext
_ = lambda x: gettext.ldgettext("anaconda", x)

import installmethod
import yuminstall

import logging
log = logging.getLogger("anaconda")

# custom installs are easy :-)
class InstallClass(BaseInstallClass):
    # name has underscore used for mnemonics, strip if you dont need it
    id = "enterprise"
    name = N_("Oracle Linux Server")
    _description = N_("The default installation of %s is a basic server install. "
                     "You can optionally select a different set of software "
                     "now.")
    _descriptionFields = (productName,)
    sortPriority = 10001

    bootloaderTimeoutDefault = 5
    bootloaderExtraArgs = ["crashkernel=auto"] 

    tasks = [(N_("Basic Server"),
              ["base", "client-mgmt-tools", "console-internet", "core", "debugging",
               "directory-client", "hardware-monitoring", "java-platform",
               "large-systems", "network-file-system-client",
               "performance", "perl-runtime", "server-platform"]),
             (N_("Database Server"),
              ["base", "client-mgmt-tools", "console-internet", "core", "debugging",
               "directory-client", "hardware-monitoring", "java-platform",
               "large-systems", "network-file-system-client",
               "performance", "perl-runtime", "server-platform",
               "mysql-client", "mysql", "postgresql-client", "postgresql",
               "system-admin-tools"]),
             (N_("Web Server"),
              ["base", "client-mgmt-tools", "console-internet", "core", "debugging",
               "directory-client", "java-platform",
               "network-file-system-client",
               "performance", "perl-runtime", "server-platform",
               "web-server", "web-servlet", "php", "turbogears",
               "mysql-client", "postgresql-client"]),
             (N_("Identity Management Server"),
              ["base", "client-mgmt-tools", "console-internet", "core", "debugging",
               "directory-client", "java-platform",
               "network-file-system-client",
               "performance", "perl-runtime", "server-platform",
               "identity-management-server"]),
             (N_("Virtualization Host"),
              ["base", "client-mgmt-tools", "console-internet", "core", "debugging",
               "directory-client", "hardware-monitoring", "java-platform",
               "large-systems", "network-file-system-client",
               "performance", "perl-runtime", "server-platform",
               "virtualization", "virtualization-client",
               "virtualization-platform"]),
             (N_("Desktop"),
              ["base", "client-mgmt-tools", "core", "debugging",
               "directory-client", "java-platform", "network-file-system-client",
               "perl-runtime", "server-platform",
               "fonts", "print-client",
               "basic-desktop", "desktop-debugging", "desktop-platform",
               "general-desktop", "graphical-admin-tools", "input-methods",
               "legacy-x", "x11", "internet-browser",
               "remote-desktop-clients"]),
             (N_("Software Development Workstation"),
              ["base", "client-mgmt-tools", "core", "debugging",
               "directory-client", "java-platform", "network-file-system-client",
               "performance", "perl-runtime", "server-platform",
               "fonts", "print-client",
               "basic-desktop", "desktop-debugging", "desktop-platform",
               "general-desktop", "graphical-admin-tools", "input-methods",
               "legacy-x", "x11", "internet-browser", "graphics", "emacs", "tex",
               "remote-desktop-clients", "virtualization", "virtualization-client",
               "virtualization-platform",
               "desktop-platform-devel", "development", "eclipse",
               "server-platform-devel", "technical-writing",
               "additional-devel"]),
             (N_("Minimal"),
              ["core"])]

    def getBackend(self):
        if flags.livecdInstall:
            import livecd
            return livecd.LiveCDCopyBackend
        else:
            return yuminstall.YumBackend

    def setInstallData(self, anaconda):
        BaseInstallClass.setInstallData(self, anaconda)
        BaseInstallClass.setDefaultPartitioning(self, anaconda.id.storage,
                                                anaconda.platform)

    def setGroupSelection(self, anaconda):
        grps = anaconda.backend.getDefaultGroups(anaconda)
        map(lambda x: anaconda.backend.selectGroup(x), grps)

    def setSteps(self, anaconda):
        BaseInstallClass.setSteps(self, anaconda)
        anaconda.dispatch.skipStep("partition")

    # for rhel, we're putting the metadata under productpath
    def getPackagePaths(self, uri):
        if not type(uri) == types.ListType:
            uri = [uri,]
        return {productName: uri}

    def productMatches(self, oldprod):
        if oldprod is None:
            return False

        if oldprod.startswith(productName):
            return True

        productUpgrades = {
            "Oracle Linux Server": ("Oracle Linux Server",
                             "Red Hat Linux Advanced Server",
                             "Red Hat Linux Advanced Workstation",
                             "Red Hat Enterprise Linux",
                             "Red Hat Enterprise Linux AS",
                             "Red Hat Enterprise Linux ES",
                             "Red Hat Enterprise Linux WS"),
            "Red Hat Enterprise Linux AS": ("Red Hat Linux Advanced Server", ),
            "Red Hat Enterprise Linux WS": ("Red Hat Linux Advanced Workstation",),
            # FIXME: this probably shouldn't be in a release...
            "Red Hat Enterprise Linux": ("Red Hat Linux Advanced Server",
                                         "Red Hat Linux Advanced Workstation",
                                         "Red Hat Enterprise Linux AS",
                                         "Red Hat Enterprise Linux ES",
                                         "Red Hat Enterprise Linux WS"),
            "Red Hat Enterprise Linux Server": ("Red Hat Enterprise Linux AS",
                                                "Red Hat Enterprise Linux ES",
                                                "Red Hat Enterprise Linux WS",
                                                "Red Hat Enterprise Linux"),
            "Red Hat Enterprise Linux Client": ("Red Hat Enterprise Linux WS",
                                                "Red Hat Enterprise Linux Desktop",
                                                "Red Hat Enterprise Linux"),
        }

        if productUpgrades.has_key(productName):
            acceptable = productUpgrades[productName]
        else:
            acceptable = ()

        for p in acceptable:
            if oldprod.startswith(p):
                return True

        return False

    def versionMatches(self, oldver):
        oldMajor = oldver.split(".")[0]
        newMajor = productVersion.split(".")[0]

        return oldMajor == newMajor

    def __init__(self, expert):
        BaseInstallClass.__init__(self, expert)
